//
//  oh_hw1_fmViewController.m
//  oh_hw1_fm
//
//  Created by Jieun Oh on 1/7/10.
//  Copyright Stanford University 2010. All rights reserved.
//

#import "oh_hw1_fmViewController.h"
#import "fm.h"
#import "mo_audio.h"
#import "mo_accel.h"
#import <math.h>


#define SRATE 44100
#define FRAMESIZE 128
#define NUMCHANNELS 2

bool g_play = true; 


void fmCallback (Float32 * buffer, UInt32 framesize, void* userData)
{
	fm *f = fm::instance();

	static float phase_c = 0.0; // 440.0/SRATE;
	static float phase_m = 0.0; // g_fm/SRATE;
	
	double deviation;
	double modulated;
	
	if (g_play) {
		for (int i=0; i<framesize; i++)
		{
			buffer[2*i] = buffer[2*i+1] = sin(2.0*M_PI*phase_c);
			
			deviation =  (f->getModIndex()) * (f->getFMod());               // index_of_modulation * modulator_freq
			modulated =  deviation * sin(2.0*M_PI*phase_m);
			
			phase_c += (f->getFCarr() + modulated)/SRATE;
			phase_m += (f->getFMod())/SRATE;
	
			
			if (phase_c > 1.0f) 
				phase_c -= 1.0f;
		
			if (phase_m > 1.0f) 
				phase_m -= 1.0f; 
		}
	}
	
	else
	{
		for (int i=0; i<framesize; i++)
		{
			buffer[2*i] = buffer[2*i+1] = 0.0;
		}
	}

		
}


// Implement audio callback here
void audioCallback( Float32 * buffer, UInt32 framesize, void* userData)
{	
	fmCallback( buffer, framesize, userData);
}


// Implement touch callback here
void accelCallback( double x, double y, double z, void *data)
{
	// NSLog(@"Accel!");
	// x,y,z are approx. +/- 5 max (can print out values to see)
	
	fm *f = fm::instance();

	AandF * aandf = (AandF *) data;
	UILabel *myAccelX = aandf->xa;
	UILabel *myAccelY = aandf->ya;
	UILabel *myCarrierFreq = aandf->cf;
	UILabel *myModulatorFreq = aandf->mf;
	
	[myAccelX setText:[NSString stringWithFormat:@"%f", x]];
	[myAccelY setText:[NSString stringWithFormat:@"%f", y]];
	
	
	f->setFCarr( 440.0 + 100*(float)x );
	f->setFMod( 40.0 + 20*(float)y );

	// NSLog(@"FCarr: %f", f->getFCarr());
	// NSLog(@"FMod: %f", f->getFMod());
	
	[myCarrierFreq setText:[NSString stringWithFormat:@"%f", f->getFCarr()]];
	[myModulatorFreq setText:[NSString stringWithFormat:@"%f", f->getFMod()]];
}



@implementation oh_hw1_fmViewController

@synthesize myAccelX;
@synthesize myAccelY;
@synthesize myCarrierFreq;
@synthesize myModulatorFreq;
@synthesize myModIndex;
@synthesize mySlider;
@synthesize myButton;

-(IBAction)moveSlider:(UISlider *)sender
{
	// NSLog(@"slider value: %f", sender.value);
	
	fm *f = fm::instance();
	f->setModIndex( (float) sender.value);
	[myModIndex setText:[NSString stringWithFormat:@"%f", sender.value]];
	//NSLog(@"in fm, test getModIndex: %f", f->getModIndex());
}

-(IBAction)moveSwitch: (UISwitch *) sender
{
	fm *f = fm::instance();
	f->setContinuous(sender.on);
	// NSLog(@"test getContinuous: %d", f->isContinuous());
	g_play = sender.on; 
	myButton.hidden = f->isContinuous();
}

-(IBAction)triggerSoundOn: (UIButton *) sender
{
	g_play = true;
}


-(IBAction)triggerSoundOff: (UIButton *) sender
{
	g_play = false;
}


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	// init audio
	NSLog(@"Initializing Audio");
	
	// init the audio layer
	bool result = MoAudio::init(SRATE, FRAMESIZE, NUMCHANNELS);
	
	if (!result)
	{
		NSLog(@"cannot initialize real-time audio!");
		return;
	}
	
	// start the audio layer, registering a callback method
	result = MoAudio::start( audioCallback, NULL);
	if (!result)
	{
		NSLog(@"cannot start real-time audio!");
		return;
	}
	
	aandf.xa = myAccelX;
	aandf.ya = myAccelY;
	aandf.cf = myCarrierFreq;
	aandf.mf = myModulatorFreq;
	
	// init and register accel callback
	MoAccel::instance()->addCallback( accelCallback, &aandf );

	myButton.hidden = true; 
	// [mySlider setValue:f->getModIndex()];
}



/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

@end
